@@ -22,11 +22,11 @@
 
 #define CHAR_WIDTH 5
 #define CHAR_HEIGHT 7
-#define VERSION "2.8"
+#define VERSION "2.9"
 #define _MULTI_THREADED
 
-typedef enum {NONE, ALARM, TIMER, TIMER_DONE, CHRONO_PAUSED, CHRONO_ACTIVE} modeType;
-typedef enum {BELL, COMMAND} actionType;
+typedef enum {NONE, ALARM, TIMER, TIMER_PAUSED, TIMER_DONE, CHRONO, CHRONO_PAUSED} modeType;
+typedef struct {int bell, command;} actionType;
 typedef enum {OUT, IN, RETURN} configState;
 
 /*******************************************************************************
@@ -113,8 +113,8 @@ int main(int argc, char *argv[])
   updateMain();
   updateACT(); 
 
-  if (hour == 0 && min == 0 && sec == 0) 
-    timeSetToZero = 1;
+  //  if (hour == 0 && min == 0 && sec == 0) 
+  //    timeSetToZero = 1;
 
   while (1) 
   {
@@ -137,7 +137,7 @@ int main(int argc, char *argv[])
 	}
 	prevSec = thisTime->tm_sec;
 	break;
-      case CHRONO_ACTIVE:
+      case CHRONO:
 	if (prevSec < thisTime->tm_sec)
 	{
 	  incrementTimer();
@@ -153,6 +153,7 @@ int main(int argc, char *argv[])
 	break;
       case NONE:
       case TIMER_DONE:
+      case TIMER_PAUSED:
       case CHRONO_PAUSED:
 	break;
     }
@@ -183,16 +184,16 @@ return 0;
  ******************************************************************************/
 void execAct()
 {
-  if (action == COMMAND)
+  if (action.command && strlen(command))
     execCommand(command);
-  else
+  if (action.bell)
   {
     printf("\07");
     fflush(stdout);
   }
 
   if (mode == TIMER)
-    mode = TIMER_DONE;	// Don't want to keep doing the Timer event
+    mode = TIMER_DONE;		// Don't want to keep doing the Timer event
   else
     mode = NONE;
 
@@ -227,11 +228,14 @@ void parseArgs(int argc, char *argv[])
 	  mode = TIMER;
 	  break;
 	case 'r':
-	  mode = CHRONO_ACTIVE;
+	  mode = CHRONO;
 	  break;
+  case 'b':
+    action.bell = 1;
+    break;
 	case 'e':
 	  strcpy(command, argv[argIndex+1]);
-	  action = COMMAND;
+	  action.command = 1;
 	  break;
 	case 't':
 	  if (argv[argIndex+1])
@@ -263,6 +267,9 @@ void parseArgs(int argc, char *argv[])
 	    hour = atoi(strtok(argv[argIndex+1], ":"));
 	    min = atoi(strtok(NULL, ":"));
 	    sec = atoi(strtok(NULL, ":"));
+
+	    if (hour == 0 && min == 0 && sec == 0) 
+	      timeSetToZero = 1;
 	  }
 	  else
 	    usage();
@@ -327,32 +334,43 @@ void processEvent(XEvent *event)
 	switch (buttonStatus) 
 	{
 	  case 0:					// center button
-	    if (mode == ALARM || mode == TIMER) 
+	    if (mode == ALARM) 
 	    {
 	      hour = min = sec = 0;
 	      updateACT();
 	    }
-	    mode = CHRONO_PAUSED;
+	    if (mode == CHRONO)
+	      mode = CHRONO_PAUSED;
+	    else if (mode == TIMER)
+	      mode = TIMER_PAUSED;
 	    updateMain();
 	    break;
 	  case 1:					// left arrow button
-	    mode = CHRONO_PAUSED;
+	    if (mode == CHRONO)
+	      mode = CHRONO_PAUSED;
+	    else if (mode == TIMER)
+	      mode = TIMER_PAUSED;
 	    hour = min = sec = 0;
 	    updateACT();
 	    updateMain();
 	    break;
 	  case 2:					// right arrow button
-	    if (mode == ALARM || mode == TIMER)
+	    if (mode == ALARM)
 	    {
 	      hour = min = sec = 0;
 	      updateACT();
+	      timeSetToZero = 0;
+	      mode = CHRONO;
 	    }
-	    timeSetToZero = 0;
-	    mode = CHRONO_ACTIVE;
+	    else if (mode == TIMER || mode == TIMER_PAUSED)
+	      mode = TIMER;
+	    else 
+	      mode = CHRONO;
+
 	    updateMain();
 	    break;
 	  case 3:					// main area
-	    if (configSt != IN) // Dont want to spawn multiple config threads
+	    if (configSt != IN) 	// Dont want to spawn multiple config threads
 	    { 
 	      threadId = pthread_create(&thread, NULL, configure, NULL);
 	      configSt = IN;
@@ -373,7 +391,7 @@ void processEvent(XEvent *event)
  ******************************************************************************/
 void usage(void)
 {
-  fprintf(stderr, "\nWMTimer - Josh King <jking@dwave.net>\n\n");
+  fprintf(stderr, "\nWMTimer - Josh King <wmtimer@darkops.net>\n\n");
 
   fprintf(stderr, "usage: %s -[a|c|r] -t <hh:mm:ss> -e <command>\n\n", myName);
 
@@ -382,7 +400,8 @@ void usage(void)
   fprintf(stderr, "    -c     countdowntimer mode, wmtimer will beep/exec\n");
   fprintf(stderr, "	         command when specified time reaches 0 \n");
   fprintf(stderr, "    -color <color> as a word or as rgb:RR/GG/BB\n");
-  fprintf(stderr, "    -e     <command> system bell is default\n");
+  fprintf(stderr, "    -b     beep\n");
+  fprintf(stderr, "    -e     <command> exec command\n");
   fprintf(stderr, "    -r     start in chronograph mode\n");
   fprintf(stderr, "    -t     <hh:mm:ss>\n");
   fprintf(stderr, "    -h     this help screen\n");
@@ -398,7 +417,7 @@ void usage(void)
  ******************************************************************************/
 void decrementTimer()
 {
-  if (!(hour == 0 && min == 0 && sec == 0)) // Don't want to go past 0:0:0
+  if (!(hour == 0 && min == 0 && sec == 0)) 	// Don't want to go past 0:0:0
     sec--;
   if (sec == -1)
   {
@@ -524,11 +543,12 @@ void updateMain()
     case ALARM:
       blitString("ALARM:", 13, 21);
       break;
+    case TIMER_PAUSED:
     case TIMER:
       blitString("TIMER:", 13, 21);
       break;
     case CHRONO_PAUSED:
-    case CHRONO_ACTIVE:
+    case CHRONO:
       blitString("CHRONO:", 12, 21);
       break;
     default:
@@ -547,21 +567,33 @@ void callback(GtkWidget * widget, gpointer data)
     tmpMode = ALARM;
   else if ((char *) data == "timer_button")
     tmpMode = TIMER;
+  else if ((char *) data == "chrono_button")
+    tmpMode = CHRONO;
   else if ((char *) data == "bell_button")
   {
-    tmpAction = BELL;
-    gtk_entry_set_text(GTK_ENTRY (entry), "");
-    gtk_entry_set_editable(GTK_ENTRY (entry), FALSE);
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+      tmpAction.bell = 1;
+    else
+      tmpAction.bell = 0;
   }
   else if ((char *) data == "command_button")
   {
-    tmpAction = COMMAND;
-    gtk_entry_set_editable(GTK_ENTRY (entry), TRUE);
-    gtk_entry_set_text(GTK_ENTRY (entry), command);
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
+    {
+      tmpAction.command = 1;
+      gtk_entry_set_editable(GTK_ENTRY (entry), TRUE);
+      gtk_entry_set_text(GTK_ENTRY (entry), command);
+    }
+    else 
+    {
+      tmpAction.command = 0;
+      gtk_entry_set_text(GTK_ENTRY (entry), "");
+      gtk_entry_set_editable(GTK_ENTRY (entry), FALSE);
+    }
   }
   else if ((char *) data == "ok")
   {
-    if (tmpAction == COMMAND)
+    if (tmpAction.command)
       strcpy(command, gtk_entry_get_text(GTK_ENTRY (entry)));
 
     hour = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (spinner1));
@@ -571,13 +603,14 @@ void callback(GtkWidget * widget, gpointer data)
 
     // If users presses 'ok' and we have not explicitly set the mode, 
     // then it was left on ALARM
-//    if (!tmpMode)
-//      tmpMode = 1;
+    if (!tmpMode)
+      tmpMode = ALARM;
 
     mode = tmpMode;
     // need to reset so that it can default to ALARM if timer not specified 
-//    tmpMode = NONE;
-    action = tmpAction;
+    tmpMode = NONE;
+    action.bell = tmpAction.bell;
+    action.command = tmpAction.command;
     configSt = RETURN;
   }
   else if (!strcmp((char *) data, "clear"))
@@ -619,6 +652,9 @@ void *configure(void *arg)
   GtkWidget *window;
   GtkWidget *frame;
   GtkWidget *button;
+  GtkWidget *alarm_button;
+  GtkWidget *timer_button;
+  GtkWidget *chrono_button;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *sub_vbox;
@@ -652,24 +688,38 @@ void *configure(void *arg)
   gtk_container_add (GTK_CONTAINER (frame), box2);
 
   // Create Alarm radio button
-  button = gtk_radio_button_new_with_label (NULL, "Alarm");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+  alarm_button = gtk_radio_button_new_with_label (NULL, "Alarm");
+  gtk_signal_connect (GTK_OBJECT (alarm_button), "clicked",
       GTK_SIGNAL_FUNC (callback), (gpointer) "alarm_button");
-  gtk_box_pack_start (GTK_BOX (box2), button, FALSE, FALSE, 2);
-  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (box2), alarm_button, FALSE, FALSE, 2);
+  gtk_widget_show (alarm_button);
 
   // Create Timer radio button
-  button = gtk_radio_button_new_with_label (gtk_radio_button_group
-      (GTK_RADIO_BUTTON (button)), "Timer");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+  timer_button = gtk_radio_button_new_with_label (gtk_radio_button_group
+      (GTK_RADIO_BUTTON (alarm_button)), "Timer");
+  gtk_signal_connect (GTK_OBJECT (timer_button), "clicked",
       GTK_SIGNAL_FUNC (callback), (gpointer) "timer_button");
-  gtk_box_pack_start (GTK_BOX (box2), button, FALSE, FALSE, 2);
-  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (box2), timer_button, FALSE, FALSE, 2);
+  gtk_widget_show (timer_button);
+  gtk_widget_show (box2);
+
+  // Create Chrono radio button
+  chrono_button = gtk_radio_button_new_with_label (gtk_radio_button_group
+      (GTK_RADIO_BUTTON (alarm_button)), "Chrono");
+  gtk_signal_connect (GTK_OBJECT (chrono_button), "clicked",
+      GTK_SIGNAL_FUNC (callback), (gpointer) "chrono_button");
+  gtk_box_pack_start (GTK_BOX (box2), chrono_button, FALSE, FALSE, 2);
+  gtk_widget_show (chrono_button);
   gtk_widget_show (box2);
 
   // If we are in timer mode, set toggle accordingly else default to alarm 
-  if (mode == TIMER || mode == TIMER_DONE)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+  // Set the button corresponding to the current mode
+  if (mode == TIMER || mode == TIMER_PAUSED || mode == TIMER_DONE)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (timer_button), TRUE);
+  else if (mode == CHRONO || mode == CHRONO_PAUSED)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chrono_button), TRUE);
+  else
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (alarm_button), TRUE);
 
   // Create frame for the time 
   frame = gtk_frame_new ("Time");
@@ -680,7 +730,7 @@ void *configure(void *arg)
   gtk_container_add (GTK_CONTAINER (frame), box2);
 
   // Create hours spinner
-  adj = (GtkAdjustment *) gtk_adjustment_new (hour, 0.0, 24.0, 1.0, 2.0, 0.0);
+  adj = (GtkAdjustment *) gtk_adjustment_new (hour, 0.0, 23.0, 1.0, 2.0, 0.0);
   spinner1 = gtk_spin_button_new (adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner1), TRUE);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinner1), TRUE);
@@ -735,16 +785,19 @@ void *configure(void *arg)
   box2 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (sub_vbox), box2, TRUE, TRUE, 2);
 
-  // Create Bell radio button
-  button = gtk_radio_button_new_with_label (NULL, "System Bell");
+  // Create Bell check button
+  button = gtk_check_button_new_with_label ("System Bell");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
       GTK_SIGNAL_FUNC (callback), (gpointer) "bell_button");
   gtk_box_pack_start (GTK_BOX (box2), button, FALSE, FALSE, 2);
   gtk_widget_show (button);
+  
+  // If bell mode is active, toggle the button
+  if (action.bell)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
-  // Create Command radio button
-  button = gtk_radio_button_new_with_label (gtk_radio_button_group
-      (GTK_RADIO_BUTTON (button)), "Command");
+  // Create Command check button
+  button = gtk_check_button_new_with_label ("Command");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
       GTK_SIGNAL_FUNC (callback), (gpointer) "command_button");
   gtk_box_pack_start (GTK_BOX (box2), button, FALSE, FALSE, 2);
@@ -771,8 +824,9 @@ void *configure(void *arg)
   gtk_widget_show (entry);
   gtk_widget_show (box2);
 
-  // If we are in command mode, allow user to enter a command
-  if (action == COMMAND)
+  // If command mode is active, toggle the button and allow user to enter a 
+  // command
+  if (action.command)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
   box2 = gtk_hbox_new (FALSE, 0);
@@ -811,3 +865,4 @@ void *configure(void *arg)
 
   return 0;
 }
+
